"""Utilities to express deployment.xml using the VT (Vespa Tag) syntax, which
maps python functions directly to xml tags. This allows you to express any deployment.xml-configuration
in python.

This mirrors the approach used for query profile configuration in
`vespa.configuration.query_profiles` and allows users to build a deployment
descriptor directly in Python, ensuring the top-level tag is `deployment`.

Example:

    from vespa.configuration.deployment import (
        deployment, prod, region
    )
    from vespa.package import ApplicationPackage

    dep_cfg = deployment(
        prod(
            region("aws-us-east-1c"),
            region("aws-us-west-2a")
        ),
        version="1.0"
    )

    app_package = ApplicationPackage(name="myapp", deployment_config=dep_cfg)
    # app_package.deployment_to_text now returns the expected deployment.xml content
    # and will write this to deployment.xml-file on deployment (or if app_package.to_files() is called explicitly)
"""

from typing import List
from vespa.configuration.vt import VT, create_tag_function, voids

# Tags used (subset guided by reference examples). Add more as needed.
# NB! If you modify this, please run 'python vespa/configuration/generate_pyi.py' to regenerate the .pyi stub files.
deployment_tags: List[str] = [
    "deployment",
    "instance",
    "prod",
    "region",
    "block-change",
    "delay",
    "parallel",
    "steps",
    "endpoints",
    "endpoint",
    "staging",
    "upgrade",
    "test",
    "dev",
    "tester",
    "nodes",
    "allow",
    "bcp",
    "group",
]

# Fail fast on duplicates
_dup = {t for t in deployment_tags if deployment_tags.count(t) > 1}
if _dup:
    raise ValueError(f"Tags duplicated in deployment_tags: {_dup}")

generated_deployment_tags = set()
g = globals()
for _tag in deployment_tags:
    sanitized_name = VT.sanitize_tag_name(_tag)
    generated_deployment_tags.add(sanitized_name)
    g[sanitized_name] = create_tag_function(_tag, _tag in voids)


class DeploymentItem:
    """Wrapper around a VT structure representing a full deployment.xml tree.

    Ensures that the provided VT has a `deployment` top-level tag and exposes
    the rendered XML via the `xml` property.
    """

    def __init__(self, root: VT):
        if not isinstance(root, VT):  # defensive
            raise TypeError(
                f"DeploymentItem expects a VT root element, got {type(root).__name__}"
            )
        if root.tag != VT.sanitize_tag_name("deployment"):
            raise ValueError(
                "Top level tag for deployment configuration must be 'deployment'"
            )
        self.root = root

    @classmethod
    def from_vt(cls, vt_obj: VT) -> "DeploymentItem":
        return cls(vt_obj)

    def to_xml(self) -> str:
        return str(self.root.to_xml()).strip()

    def __repr__(self) -> str:  # pragma: no cover
        return f"DeploymentItem(root={self.root!r})"



# Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.

# This script should modify top level pyproject.toml to update the version of pyvespa

import sys
import re
from pathlib import Path
import argparse

PYPROJECT_TOML_PATH = Path(__file__).parent.parent.parent / "pyproject.toml"


def update_version(new_version: str):
    # Update version in pyproject.toml
    with open(PYPROJECT_TOML_PATH, "r") as f:
        content = f.read()
    new_content = re.sub(r'version = ".*"', f'version = "{new_version}"', content)
    with open(PYPROJECT_TOML_PATH, "w") as f:
        f.write(new_content)
    print(f"Updated version to {new_version}")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Update pyvespa version")
    parser.add_argument(
        "-v", "--version", type=str, help="New version to set", required=True
    )
    args = parser.parse_args()
    update_version(args.version)
    sys.exit(0)



# Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.

from vespa.application import Vespa

import argparse
import urllib.parse

from vespa.evaluation import VespaNNGlobalFilterHitratioEvaluator


def query_from_get_string(get_query):
    url = urllib.parse.urlparse(get_query)
    assert url.path == "/search/"
    parsed_query = urllib.parse.parse_qs(url.query)
    query = {}
    for key in parsed_query.keys():
        query[key] = parsed_query[key][0]

    assert "yql" in query
    return query


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        prog="Vespa ANN Hit-Ratio Tool",
        description="Determines hit ratios of ANN queries.",
    )
    parser.add_argument("url", help="URL.")
    parser.add_argument("port", help="Port.")
    parser.add_argument(
        "query_file", help="File containing queries in GET format (one per line)."
    )
    parser.add_argument("--silent", action="store_true")
    parser.set_defaults(silent=False)
    parser.add_argument("--cert")
    parser.set_defaults(cert=None)
    parser.add_argument("--key")
    parser.set_defaults(key=None)
    args = parser.parse_args()

    app = Vespa(url=args.url, port=args.port, cert=args.cert, key=args.key)

    # Read query file with get queries
    with open(args.query_file) as file:
        get_queries = file.read().splitlines()

    # Parse get queries
    queries = list(map(query_from_get_string, get_queries))

    # Compute hit ratios
    if not args.silent:
        print("Determining hit ratios")
    hitratio_evaluator = VespaNNGlobalFilterHitratioEvaluator(queries, app)
    hitratios = hitratio_evaluator.run()

    # Print hit ratios
    if args.silent:
        for hitratio in hitratios:
            print(hitratio)
    else:
        print("Hit ratios:")
        for hitratio in hitratios:
            print(f"  {hitratio}")



# Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.

"""
Vespa evaluation module.

This module provides tools for evaluating and benchmarking Vespa applications.
"""

# Import all public symbols from _base module (original evaluation.py content)
from vespa.evaluation._base import *  # noqa: F401, F403

# Lazy import for MTEB-related classes to avoid requiring mteb as a dependency
# when not using MTEB functionality


def __getattr__(name):
    """Lazy import for optional MTEB dependencies."""
    if name in ("VespaMTEBApp", "VespaMTEBEvaluator"):
        from vespa.evaluation._mteb import VespaMTEBApp, VespaMTEBEvaluator

        if name == "VespaMTEBApp":
            return VespaMTEBApp
        return VespaMTEBEvaluator
    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")



from fastcore.utils import tuplify
import types
from xml.sax.saxutils import escape
from fastcore.utils import patch
import xml.etree.ElementTree as ET

# If the vespa tags correspond to reserved Python keywords or commonly used names,
# they are replaced with the following:
replace_reserved = {
    "type": "type_",
    "class": "class_",
    "for": "for_",
    "time": "time_",
    "io": "io_",
    "from": "from_",
    "match": "match_",
    "test": "test_",
}
restore_reserved = {v: k for k, v in replace_reserved.items()}


class VT:
    "A 'Vespa Tag' structure, containing `tag`, `children`, and `attrs`"

    @staticmethod
    def sanitize_tag_name(tag: str) -> str:
        "Convert invalid tag names (with '-') to valid Python identifiers (with '_')"
        replaced = tag.replace("-", "_")
        return replace_reserved.get(replaced, replaced)

    def __init__(
        self,
        tag: str,
        cs: tuple,
        attrs: dict = None,
        void_=False,
        replace_underscores: bool = True,
        **kwargs,
    ):
        assert isinstance(cs, tuple)
        self.tag = self.sanitize_tag_name(tag)  # Sanitize tag name
        self.children, self.attrs = cs, attrs or {}
        self.void_ = void_
        self.replace_underscores = replace_underscores

    def __setattr__(self, k, v):
        if k.startswith("__") or k in (
            "tag",
            "children",
            "attrs",
            "void_",
            "replace_underscores",
        ):
            return super().__setattr__(k, v)
        self.attrs[k.lstrip("_").replace("_", "-")] = v

    def __getattr__(self, k):
        if k.startswith("__"):
            raise AttributeError(k)
        return self.get(k)

    @property
    def list(self):
        return [self.tag, self.children, self.attrs]

    def get(self, k, default=None):
        return self.attrs.get(k.lstrip("_").replace("_", "-"), default)

    def restore_tag_name(
        self,
    ) -> str:
        "Restore sanitized tag names back to the original names for XML generation"
        restored = restore_reserved.get(self.tag, self.tag)
        if self.replace_underscores:
            return restored.replace("_", "-")
        return restored

    def __repr__(self):
        return f"{self.tag}({self.children},{self.attrs})"

    def __add__(self, b):
        self.children = self.children + tuplify(b)
        return self

    def __getitem__(self, idx):
        return self.children[idx]

    def __iter__(self):
        return iter(self.children)


def attrmap(o):
    """This maps the attributes that we don't want to be Python keywords or commonly used names to the replacement names."""
    # Handle reserved attribute names by checking if they end with underscore and are in our mapping
    if o in restore_reserved:
        o = restore_reserved[o]
    else:
        o = dict(_global="global").get(o, o)
    return o.lstrip("_").replace("_", "-")


def valmap(o):
    """Convert values to the string representation for xml. integers to strings and booleans to 'true' or 'false'"""
    if isinstance(o, bool):
        return str(o).lower()
    elif isinstance(o, int):
        return str(o)
    return o if isinstance(o, str) else " ".join(map(str, o))


def _flatten_tuple(tup):
    result = []
    for item in tup:
        if isinstance(item, tuple):
            result.extend(item)
        else:
            result.append(item)
    return tuple(result)


def _preproc(c, kw, attrmap=attrmap, valmap=valmap):
    """
    Preprocess the children and attributes of a VT structure.

    :param c: Children of the VT structure
    :param kw: Attributes of the VT structure
    :param attrmap: Dict to map attribute names
    :param valmap: Dict to map attribute values

    :return: Tuple of children and attributes
    """

    # If the children are a single generator, map, or filter, convert it to a tuple
    if len(c) == 1 and isinstance(c[0], (types.GeneratorType, map, filter)):
        c = tuple(c[0])
    # Create the attributes dictionary by mapping the keys and values
    # TODO: Check if any of Vespa supported attributes are camelCase
    attrs = {attrmap(k.lower()): valmap(v) for k, v in kw.items() if v is not None}
    return _flatten_tuple(c), attrs


# General XML tag creation function
def vt(
    tag: str,
    *c,
    void_: bool = False,
    attrmap: callable = attrmap,
    valmap: callable = valmap,
    replace_underscores: bool = True,
    **kw,
):
    "Create a VT structure with `tag`, `children` and `attrs`"
    # NB! fastcore.xml uses tag.lower() for tag names. This is not done here.
    return VT(
        tag,
        *_preproc(c, kw, attrmap=attrmap, valmap=valmap),
        void_=void_,
        replace_underscores=replace_underscores,
    )


# XML void tags (self-closing)
# TODO: Add self-closing tags for Vespa configuration
voids = set("".split())


def Xml(*c, version="1.0", encoding="UTF-8", **kwargs) -> VT:
    "A top level XML tag, with `encoding` and children `c`"
    res = vt("?xml", *c, version=version, encoding=encoding, void_="?")
    return res


# Replace the 'partial' based tag creation
def create_tag_function(tag, void_):
    def tag_function(*c, **kwargs):
        return vt(tag, *c, void_=void_, **kwargs)

    tag_function.__name__ = VT.sanitize_tag_name(tag)  # Assigning sanitized tag name
    return tag_function


def vt_escape(s):
    return escape(s) if isinstance(s, str) else s


def _to_attr(k, v):
    if isinstance(v, bool):
        return f"{k}" if v else ""
    return f'{k}="{escape(str(v))}"'


def _to_xml(elm, lvl, indent, do_escape):
    esc_fn = vt_escape if do_escape else lambda s: s
    nl = "\n" if indent else ""
    sp = " " * lvl if indent else ""

    if elm is None:
        return ""

    if isinstance(elm, tuple):
        return (
            f"{nl}".join(
                _to_xml(o, lvl=lvl, indent=indent, do_escape=do_escape) for o in elm
            )
            + nl
        )

    if not isinstance(elm, VT):
        # Ensure text content is compact (no trailing newline unless indent=True)
        return f"{esc_fn(str(elm).strip())}{nl if indent else ''}"

    tag, cs, attrs = elm.list

    stag = elm.restore_tag_name()

    # Prepare the attribute string only once
    attr_str = ""
    if attrs:
        attr_str = " " + " ".join(_to_attr(k, v) for k, v in attrs.items())

    # Handle void (self-closing) tags
    if elm.void_:
        if isinstance(elm.void_, str):
            return f"{sp}<{stag}{attr_str} {elm.void_}>{nl}"
        return f"{sp}<{stag}{attr_str} />{nl}"

    # Handle non-void tags with children or no children
    if cs:
        # Handle the case where children are text or elements
        res = f"{sp}<{stag}{attr_str}>"

        # If the children are just text or int, don't introduce newlines
        if len(cs) == 1 and (isinstance(cs[0], str) or isinstance(cs[0], int)):
            if isinstance(cs[0], int):
                cs = str(cs[0])
            res += f"{esc_fn(cs[0].strip())}</{stag}>{nl if indent else ''}"
        else:
            # If there are multiple children, properly indent them
            res += f"{nl if indent else ''}"
            res += "".join(
                _to_xml(c, lvl=lvl + 2, indent=indent, do_escape=do_escape) for c in cs
            )
            res += f"{sp}</{stag}>{nl if indent else ''}"

        return Safe(res)
    else:
        # Non-void tag without children
        return f"{sp}<{stag}{attr_str}></{stag}>{nl if indent else ''}"


def to_xml(elm, lvl=0, indent: bool = True, do_escape: bool = True):
    "Convert `vt` element tree into an XML string"
    return Safe(_to_xml(elm, lvl, indent, do_escape=do_escape))


VT.to_xml = to_xml


class Safe(str):
    def __html__(self):
        return self


def highlight(s, lang="xml"):
    "Markdown to syntax-highlight `s` in language `lang`"
    return f"```{lang}\n{to_xml(s)}\n```"


def showtags(s):
    return f"""<code><pre>{escape(to_xml(s))}</code></pre>"""


VT._repr_markdown_ = highlight


@patch
def __call__(self: VT, *c, **kw):
    c, kw = _preproc(c, kw)
    if c:
        self = self + c
    if kw:
        self.attrs = {**self.attrs, **kw}
    return self


def canonicalize(element):
    """Recursively sort attributes and children to canonicalize the element."""
    # Sort attributes
    if element.attrib:
        element.attrib = dict(sorted(element.attrib.items()))
    # Sort children by tag and text
    children = list(element)
    for child in children:
        canonicalize(child)
    element[:] = sorted(children, key=lambda e: (e.tag, (e.text or "").strip()))
    # Strip whitespace from text and tail
    if element.text:
        element.text = element.text.strip()
    if element.tail:
        element.tail = element.tail.strip()


def elements_equal(e1, e2):
    """Compare two elements for equality."""
    if e1.tag != e2.tag:
        return False
    if sorted(e1.attrib.items()) != sorted(e2.attrib.items()):
        return False
    if (e1.text or "").strip() != (e2.text or "").strip():
        return False
    if (e1.tail or "").strip() != (e2.tail or "").strip():
        return False
    if len(e1) != len(e2):
        return False
    return all(elements_equal(c1, c2) for c1, c2 in zip(e1, e2))


def compare_xml(xml_str1, xml_str2):
    """Compare two XML strings for equality."""
    try:
        tree1 = ET.ElementTree(ET.fromstring(xml_str1))
        tree2 = ET.ElementTree(ET.fromstring(xml_str2))
    except ET.ParseError:
        return False
    root1 = tree1.getroot()
    root2 = tree2.getroot()
    canonicalize(root1)
    canonicalize(root2)
    return elements_equal(root1, root2)



from vespa.configuration.vt import VT

def abortondocumenterror(*c, **kwargs) -> VT: ...
def accesslog(*c, **kwargs) -> VT: ...
def admin(*c, **kwargs) -> VT: ...
def adminserver(*c, **kwargs) -> VT: ...
def age(*c, **kwargs) -> VT: ...
def api_key_secret_ref(*c, **kwargs) -> VT: ...
def batching(*c, **kwargs) -> VT: ...
def binding(*c, **kwargs) -> VT: ...
def bucket_splitting(*c, **kwargs) -> VT: ...
def cache(*c, **kwargs) -> VT: ...
def certificate(*c, **kwargs) -> VT: ...
def chain(*c, **kwargs) -> VT: ...
def chunk(*c, **kwargs) -> VT: ...
def client(*c, **kwargs) -> VT: ...
def clients(*c, **kwargs) -> VT: ...
def cluster_controller(*c, **kwargs) -> VT: ...
def clustercontroller(*c, **kwargs) -> VT: ...
def clustercontrollers(*c, **kwargs) -> VT: ...
def component(*c, **kwargs) -> VT: ...
def components(*c, **kwargs) -> VT: ...
def compression(*c, **kwargs) -> VT: ...
def concurrency(*c, **kwargs) -> VT: ...
def config(*c, **kwargs) -> VT: ...
def configserver(*c, **kwargs) -> VT: ...
def configservers(*c, **kwargs) -> VT: ...
def conservative(*c, **kwargs) -> VT: ...
def container(*c, **kwargs) -> VT: ...
def content(*c, **kwargs) -> VT: ...
def coverage(*c, **kwargs) -> VT: ...
def dimensions(*c, **kwargs) -> VT: ...
def disk(*c, **kwargs) -> VT: ...
def disk_limit_factor(*c, **kwargs) -> VT: ...
def diskbloatfactor(*c, **kwargs) -> VT: ...
def dispatch(*c, **kwargs) -> VT: ...
def dispatch_policy(*c, **kwargs) -> VT: ...
def distribution(*c, **kwargs) -> VT: ...
def document(*c, **kwargs) -> VT: ...
def document_api(*c, **kwargs) -> VT: ...
def document_processing(*c, **kwargs) -> VT: ...
def document_token_id(*c, **kwargs) -> VT: ...
def documentprocessor(*c, **kwargs) -> VT: ...
def documents(*c, **kwargs) -> VT: ...
def endpoint(*c, **kwargs) -> VT: ...
def engine(*c, **kwargs) -> VT: ...
def environment_variables(*c, **kwargs) -> VT: ...
def execution_mode(*c, **kwargs) -> VT: ...
def federation(*c, **kwargs) -> VT: ...
def feeding(*c, **kwargs) -> VT: ...
def filtering(*c, **kwargs) -> VT: ...
def flush_on_shutdown(*c, **kwargs) -> VT: ...
def flushstrategy(*c, **kwargs) -> VT: ...
def gpu(*c, **kwargs) -> VT: ...
def gpu_device(*c, **kwargs) -> VT: ...
def group(*c, **kwargs) -> VT: ...
def groups_allowed_down_ratio(*c, **kwargs) -> VT: ...
def handler(*c, **kwargs) -> VT: ...
def http(*c, **kwargs) -> VT: ...
def ignore_undefined_fields(*c, **kwargs) -> VT: ...
def include(*c, **kwargs) -> VT: ...
def index(*c, **kwargs) -> VT: ...
def init_progress_time(*c, **kwargs) -> VT: ...
def initialize(*c, **kwargs) -> VT: ...
def interop_threads(*c, **kwargs) -> VT: ...
def interval(*c, **kwargs) -> VT: ...
def intraop_threads(*c, **kwargs) -> VT: ...
def io_(*c, **kwargs) -> VT: ...
def jvm(*c, **kwargs) -> VT: ...
def level(*c, **kwargs) -> VT: ...
def lidspace(*c, **kwargs) -> VT: ...
def logstore(*c, **kwargs) -> VT: ...
def maintenance(*c, **kwargs) -> VT: ...
def max_bloat_factor(*c, **kwargs) -> VT: ...
def max_concurrent(*c, **kwargs) -> VT: ...
def max_document_tokens(*c, **kwargs) -> VT: ...
def max_hits_per_partition(*c, **kwargs) -> VT: ...
def max_premature_crashes(*c, **kwargs) -> VT: ...
def max_query_tokens(*c, **kwargs) -> VT: ...
def max_tokens(*c, **kwargs) -> VT: ...
def max_wait_after_coverage_factor(*c, **kwargs) -> VT: ...
def maxage(*c, **kwargs) -> VT: ...
def maxfilesize(*c, **kwargs) -> VT: ...
def maxmemorygain(*c, **kwargs) -> VT: ...
def maxpendingbytes(*c, **kwargs) -> VT: ...
def maxpendingdocs(*c, **kwargs) -> VT: ...
def maxsize(*c, **kwargs) -> VT: ...
def maxsize_percent(*c, **kwargs) -> VT: ...
def mbusport(*c, **kwargs) -> VT: ...
def memory(*c, **kwargs) -> VT: ...
def memory_limit_factor(*c, **kwargs) -> VT: ...
def merges(*c, **kwargs) -> VT: ...
def min_active_docs_coverage(*c, **kwargs) -> VT: ...
def min_distributor_up_ratio(*c, **kwargs) -> VT: ...
def min_node_ratio_per_group(*c, **kwargs) -> VT: ...
def min_redundancy(*c, **kwargs) -> VT: ...
def min_storage_up_ratio(*c, **kwargs) -> VT: ...
def min_wait_after_coverage_factor(*c, **kwargs) -> VT: ...
def minimum(*c, **kwargs) -> VT: ...
def model(*c, **kwargs) -> VT: ...
def model_config_override(*c, **kwargs) -> VT: ...
def model_evaluation(*c, **kwargs) -> VT: ...
def models(*c, **kwargs) -> VT: ...
def native(*c, **kwargs) -> VT: ...
def niceness(*c, **kwargs) -> VT: ...
def node(*c, **kwargs) -> VT: ...
def nodes(*c, **kwargs) -> VT: ...
def normalize(*c, **kwargs) -> VT: ...
def onnx(*c, **kwargs) -> VT: ...
def onnx_execution_mode(*c, **kwargs) -> VT: ...
def onnx_gpu_device(*c, **kwargs) -> VT: ...
def onnx_interop_threads(*c, **kwargs) -> VT: ...
def onnx_intraop_threads(*c, **kwargs) -> VT: ...
def persearch(*c, **kwargs) -> VT: ...
def persistence_threads(*c, **kwargs) -> VT: ...
def pooling_strategy(*c, **kwargs) -> VT: ...
def prepend(*c, **kwargs) -> VT: ...
def processing(*c, **kwargs) -> VT: ...
def processor(*c, **kwargs) -> VT: ...
def proton(*c, **kwargs) -> VT: ...
def provider(*c, **kwargs) -> VT: ...
def prune(*c, **kwargs) -> VT: ...
def quantization(*c, **kwargs) -> VT: ...
def query(*c, **kwargs) -> VT: ...
def query_timeout(*c, **kwargs) -> VT: ...
def query_token_id(*c, **kwargs) -> VT: ...
def read(*c, **kwargs) -> VT: ...
def redundancy(*c, **kwargs) -> VT: ...
def removed_db(*c, **kwargs) -> VT: ...
def renderer(*c, **kwargs) -> VT: ...
def requestthreads(*c, **kwargs) -> VT: ...
def resource_limits(*c, **kwargs) -> VT: ...
def resources(*c, **kwargs) -> VT: ...
def retrydelay(*c, **kwargs) -> VT: ...
def retryenabled(*c, **kwargs) -> VT: ...
def route(*c, **kwargs) -> VT: ...
def search(*c, **kwargs) -> VT: ...
def searchable_copies(*c, **kwargs) -> VT: ...
def searcher(*c, **kwargs) -> VT: ...
def searchnode(*c, **kwargs) -> VT: ...
def secret_store(*c, **kwargs) -> VT: ...
def secrets(*c, **kwargs) -> VT: ...
def server(*c, **kwargs) -> VT: ...
def services(*c, **kwargs) -> VT: ...
def slobrok(*c, **kwargs) -> VT: ...
def slobroks(*c, **kwargs) -> VT: ...
def stable_state_period(*c, **kwargs) -> VT: ...
def store(*c, **kwargs) -> VT: ...
def summary(*c, **kwargs) -> VT: ...
def sync_transactionlog(*c, **kwargs) -> VT: ...
def term_score_threshold(*c, **kwargs) -> VT: ...
def threadpool(*c, **kwargs) -> VT: ...
def threads(*c, **kwargs) -> VT: ...
def time_(*c, **kwargs) -> VT: ...
def timeout(*c, **kwargs) -> VT: ...
def token(*c, **kwargs) -> VT: ...
def tokenizer_model(*c, **kwargs) -> VT: ...
def tokenizer_vocab(*c, **kwargs) -> VT: ...
def top_k_probability(*c, **kwargs) -> VT: ...
def total(*c, **kwargs) -> VT: ...
def tracelevel(*c, **kwargs) -> VT: ...
def transactionlog(*c, **kwargs) -> VT: ...
def transformer_attention_mask(*c, **kwargs) -> VT: ...
def transformer_end_sequence_token(*c, **kwargs) -> VT: ...
def transformer_input_ids(*c, **kwargs) -> VT: ...
def transformer_mask_token(*c, **kwargs) -> VT: ...
def transformer_model(*c, **kwargs) -> VT: ...
def transformer_output(*c, **kwargs) -> VT: ...
def transformer_pad_token(*c, **kwargs) -> VT: ...
def transformer_start_sequence_token(*c, **kwargs) -> VT: ...
def transformer_token_type_ids(*c, **kwargs) -> VT: ...
def transition_time(*c, **kwargs) -> VT: ...
def truncate(*c, **kwargs) -> VT: ...
def tuning(*c, **kwargs) -> VT: ...
def type_(*c, **kwargs) -> VT: ...
def unpack(*c, **kwargs) -> VT: ...
def visibility_delay(*c, **kwargs) -> VT: ...
def visitors(*c, **kwargs) -> VT: ...
def warmup(*c, **kwargs) -> VT: ...
def zookeeper(*c, **kwargs) -> VT: ...



# Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.

from vespa.application import Vespa

import argparse
import urllib.parse

from vespa.evaluation import VespaNNRecallEvaluator


def query_from_get_string(get_query):
    url = urllib.parse.urlparse(get_query)
    assert url.path == "/search/"
    parsed_query = urllib.parse.parse_qs(url.query)
    query = {}
    for key in parsed_query.keys():
        query[key] = parsed_query[key][0]

    assert "yql" in query
    return query


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        prog="Vespa ANN Recall Tool", description="Determines recall of ANN queries."
    )
    parser.add_argument("url", help="URL.")
    parser.add_argument("port", help="Port.")
    parser.add_argument(
        "query_file", help="File containing queries in GET format (one per line)."
    )
    parser.add_argument("hits", help="Number of target hits used in queries.")
    parser.add_argument("--silent", action="store_true")
    parser.set_defaults(silent=False)
    parser.add_argument("--cert")
    parser.set_defaults(cert=None)
    parser.add_argument("--key")
    parser.set_defaults(key=None)
    args = parser.parse_args()

    app = Vespa(url=args.url, port=args.port, cert=args.cert, key=args.key)

    # Read query file with get queries
    with open(args.query_file) as file:
        get_queries = file.read().splitlines()

    # Parse get queries
    queries = list(map(query_from_get_string, get_queries))

    # Compute hit ratios
    if not args.silent:
        print("Determining recalls")
    recall_evaluator = VespaNNRecallEvaluator(queries, int(args.hits), app)
    recalls = recall_evaluator.run()

    # Print hit ratios
    if args.silent:
        for recall in recalls:
            print(recall)
    else:
        print("Recalls:")
        for recall in recalls:
            print(f"  {recall}")



from pathlib import Path
import os
from lxml import etree

# Dict of filename (without extension) to RELAXNG schema (open(rb))
RELAXNG = {}
to_import = ["services", "validation-overrides"]
for schema_file in Path(os.path.dirname(__file__)).glob("*.rng"):
    if schema_file.stem in to_import:
        with open(schema_file, "rb") as fh:
            RELAXNG[schema_file.stem] = etree.RelaxNG(etree.parse(fh))
